zebkit.package("ui", function(pkg, Class) {
    pkg.events.regEvents("menuItemSelected");

    /**
     * Menu event class
     * @constructor
     * @class zebkit.ui.event.MenuEvent
     * @extends zebkit.Event
     */
    pkg.event.MenuEvent = Class(zebkit.Event, [
        function $prototype() {
            /**
             * Index of selected menu item
             * @type {Integer}
             * @attribute index
             * @readOnly
             */
            this.index = -1;

            /**
             * Selected menu item component
             * @type {zebkit.ui.Panel}
             * @attribute item
             * @readOnly
             */
            this.item = null;

            /**
             * Fill menu event with specified parameters
             * @param  {zebkit.ui.Menu} src a source of the menu event
             * @param  {Integer} index an index of selected menu item
             * @param  {zebkit.ui.Panel} item a selected menu item
             * @protected
             * @chainable
             * @method $fillWith
             */
            this.$fillWith = function(src, index, item) {
                this.source = src;
                this.index  = index;
                this.item   = item;
                return this;
            };
        }
    ]);

    var MENU_EVENT = new pkg.event.MenuEvent();

    /**
     * Show the given popup menu.
     * @param  {zebkit.ui.Panel} context  an UI component of zebkit hierarchy
     * @param  {zebkit.ui.Menu}  menu a menu to be shown
     * @for  zebkit.ui
     * @method showPopupMenu
     */
    pkg.showPopupMenu = function(context, menu) {
        context.getCanvas().getLayer(pkg.PopupLayerMix.id).add(menu);
    };

    /**
     * Menu item panel class. The component holds menu item content like caption, icon, sub-menu
     * sign elements. The area of the component is split into three parts: left, right and center.
     * Central part keeps content, left side keeps checked sign element and the right side keeps
     * sub-menu sign element.
     * @param  {String|zebkit.ui.Panel} content a menu item content string or component. Caption
     * string can encode the item id, item icon and item checked state. For instance:
     *
     *         {
     *             content:  "Test" | {zebkit.ui.Panel}
     *             checked:  {Boolean},                   // optional
     *             group  :  {zebkit.ui.Group},           // optional
     *             icon   :  "path/to/image" | {Image},   // optional
     *             handler:  {Function}                   // optional
     *             id     :  {String}                     // optional
     *         }
     *
     * @example
     *
     *
     *     // create menu item with icon and "Item 1" title
     *     var mi = new zebkit.ui.MenuItem({
     *         content: "Menu item label"
     *     });
     *
     *
     * @class zebkit.ui.MenuItem
     * @extends zebkit.ui.Panel
     * @constructor
     *
     */
    pkg.MenuItem = Class(pkg.Panel, [
        function(c) {
            this.$super();

            if (zebkit.isString(c)) {
                this.add(new this.clazz.Checkbox()).setVisible(false);
                c = new this.clazz.ImageLabel(c, null);
            } else if (zebkit.instanceOf(c, pkg.Panel) === false) {
                var ch = null;

                if (c.checked === true || c.checked === false || c.group !== undefined) {
                    ch = (c.group !== undefined) ? new this.clazz.Radiobox()
                                                 : new this.clazz.Checkbox();

                    if (c.group !== undefined) {
                        ch.setGroup(c.group);
                    }

                    ch.setValue(c.checked === undefined ? false : c.checked);
                } else {
                    ch = new this.clazz.Checkbox();
                    ch.setVisible(false);
                }

                this.add(ch);

                if (c.id !== undefined) {
                    this.setId(c.id);
                }

                if (c.handler !== undefined) {
                    this.$handler = c.handler;
                }

                if (zebkit.instanceOf(c.content, zebkit.ui.Panel)) {
                    c = c.content;
                } else if (c.icon !== undefined) {
                    c = new this.clazz.ImageLabel(c.content, c.icon);
                } else {
                    c = new this.clazz.ImageLabel(c.content, null);
                }
            } else {
                this.add(new this.clazz.Checkbox()).setVisible(false);
            }

            this.add(c);
            this.add(new this.clazz.SubImage());

            this.setEnabled(c.isEnabled);
            this.setVisible(c.isVisible);
        },

        function $clazz() {
            this.SubImage  = Class(pkg.StatePan, []);
            this.Label     = Class(pkg.Label,    []);
            this.Checkbox  = Class(pkg.Checkbox, []);
            this.Radiobox  = Class(pkg.Radiobox, []);
            this.ImageLabel  = Class(pkg.ImageLabel, []);
        },

        function $prototype() {
            /**
             * Menu item handler function.
             * @private
             * @attribute $handler
             * @readOnly
             * @type {Function}
             */
            this.$handler = null;

            /**
             * Gap between checked, content and sub menu arrow components
             * @attribute gap
             * @type {Integer}
             * @readOnly
             * @default 8
             */
            this.gap = 8;

            /**
             * Callback method that is called every time the menu item has
             * been selected.
             * @method  itemSelected
             */
            this.itemSelected = function() {
                var content = this.getContent();
                if (zebkit.instanceOf(content, pkg.Checkbox)) {
                    content.setValue(!content.getValue());
                }

                if (this.getCheck().isVisible) {
                    this.getCheck().toggle();
                }

                if (this.$handler !== null) {
                    this.$handler.call(this);
                }
            };

            /**
             * Set the menu item icon.
             * @param {String|Image} img a path to an image or image object
             * @method setIcon
             * @chainable
             */
            this.setIcon = function(img) {
                this.getContent().setImage(img);
                return this;
            };

            /**
             * Get check state component
             * @return {zebkit.ui.Panel} a check state component
             * @method getCheck
             */
            this.getCheck = function() {
                return this.kids[0];
            };

            /**
             * Get checked state of the item
             * @return {Boolean} a checked state
             * @method isChecked
             */
            this.isChecked = function() {
                return this.getCheck().isVisible && this.getCheck().getValue();
            };

            /**
             * Set group
             * @param {zebkit.ui.Group} g a group
             * @param {Boolean} [v] a value
             * @method setGroup
             */
            this.setGroup = function(g, v) {
                this.getCheck().setGroup(g);
                this.getCheck().setVisible(true);
                if (arguments.length > 1) {
                    this.getCheck().setValue(v);
                }
            };

            /**
             * Get content component
             * @return {zebkit.ui.Panel} a content component
             * @method getContent
             */
            this.getContent = function() {
                return this.kids.length > 0 ? this.kids[1] : null;
            };

            /**
             * Get menu item child component to render sub item arrow element
             * @return {zebkit.ui.Panel} a sub item arrow component
             * @method getSub
             * @protected
             */
            this.getSub = function() {
                return this.kids.length > 1 ? this.kids[2] : null;
            };

            this.activateSub = function(b) {
                var kid = this.getSub();
                kid.setState(b ? "pressed" : "*");
                if (this.parent !== null && this.parent.noSubIfEmpty === true) {
                    kid.setVisible(b);
                }
            };

            this.$getCheckSize = function() {
                var ch = this.getCheck();
                return ch === null ? { width: 0, height: 0 } : ch.getPreferredSize() ;
            };

            this.$getContentSize = function() {
                var content = this.getContent();
                return (content !== null && content.isVisible === true) ? content.getPreferredSize()
                                                                        : { width : 0, height : 0 };
            };

            this.$getSubSize = function() {
                var sub = this.getSub();
                return (sub !== null && sub.isVisible === true) ? sub.getPreferredSize()
                                                                : { width : 0, height : 0 };
            };

            this.calcPreferredSize = function (target){
                var p1 = this.$getCheckSize(),
                    p2 = this.$getContentSize(),
                    p3 = this.$getSubSize(),
                    h  = Math.max(p1.height, p2.height, p3.height),
                    w  = p1.width + p2.width + p3.width,
                    i  = -1;

                if (p1.width > 0) {
                    i++;
                }

                if (p2.width > 0) {
                    i++;
                }

                if (p3.width > 0) {
                    i++;
                }

                return { width: w + (i > 0 ? this.gap * i : 0), height: h };
            };

            this.doLayout = function(target) {
                var left    = this.getCheck(),
                    right   = this.getSub(),
                    content = this.getContent(),
                    p1      = this.$getCheckSize(),
                    p3      = this.$getSubSize(),
                    t       = target.getTop(),
                    l       = target.getLeft(),
                    eh      = target.height - t - target.getBottom(),
                    ew      = target.width  - l - target.getRight();

                if (left !== null && left.isVisible === true) {
                    left.toPreferredSize();
                    left.setLocation(l, t + Math.floor((eh - left.height)/2));
                }

                var add = (p1.width > 0 ? this.gap : 0) + p1.width;
                l  += add;
                ew -= add;

                if (right !== null && right.isVisible === true) {
                    right.toPreferredSize();
                    right.setLocation(target.width - target.getRight() - right.width,
                                      t + Math.floor((eh - right.height)/2));
                }
                ew -= ((p3.width > 0 ? this.gap : 0) + p3.width);

                if (content !== null && content.isVisible === true) {
                    content.toPreferredSize();
                    if (content.width > ew) {
                        content.setSize(ew, content.height);
                    }
                    content.setLocation(l, t + Math.floor((eh - content.height)/2));
                }
            };
        },

        /**
         * Override setParent method to catch the moment when the
         * item is inserted to a menu
         * @param {zebkit.ui.Panel} p a parent
         * @method setParent
         */
        function setParent(p) {
            this.$super(p);
            if (p !== null && p.noSubIfEmpty === true) {
                this.getSub().setVisible(false);
            }
        }
    ]).hashable();

    /**
     * Menu UI component class. The class implements popup menu UI component.
     *
     *     var m = new Menu([
     *         {
     *             content: "Menu Item 1",
     *             sub    : [
     *                 {
     *                     content: "SubMenu Checked Item 1",
     *                     checked: true
     *                 },
     *                 {
     *                     content: "SubMenu Checked Item 2",
     *                     checked: false
     *                 },
     *                 "-", // line
     *                 {
     *                     content: "SubMenu Checked Item 3",
     *                     checked: false
     *                 }
     *             ]
     *          },
     *         "Menu Item 2",
     *         "Menu Item 3"
     *      ]);
     *
     * @class zebkit.ui.Menu
     * @constructor
     * @param {Object} [list] menu items description
     * @extends zebkit.ui.CompList
     */
    pkg.Menu = Class(pkg.CompList, [
        function (d) {
            this.menus = {};

            this.$super([], zebkit.isBoolean(d) ? d : true);

            if (arguments.length > 0) {
                for(var i = 0; i < d.length; i++) {
                    var item = d[i];
                    this.add(item);
                    if (zebkit.isString(item) === false && item.sub !== undefined) {
                        var sub = item.sub;
                        this.setMenuAt(this.kids.length - 1, zebkit.instanceOf(sub, pkg.Menu) ? sub
                                                                                              : new pkg.Menu(sub));
                    }
                }
            }
        },

        function $clazz() {
            this.MenuItem = Class(pkg.MenuItem, [
                function $clazz() {
                    this.Label = Class(pkg.MenuItem.Label, []);
                }
            ]);

            this.Line = Class(pkg.Line, []);
            this.Line.prototype.$isDecorative = true;
        },

        function $prototype() {
            /**
             * Parent menu reference/
             * @attribute $parentMenu
             * @readOnly
             * @private
             * @type {zebkit.ui.Menu}
             */
            this.$parentMenu = null;

            this.canHaveFocus = true;
            this.noSubIfEmpty = false;

            /**
             * Test if the given menu item is a decorative (not selectable) menu item.
             * Menu item is considered as decorative if it has been added with addDecorative(...)
             * method or has "$isDecorative" property set to "true"
             * @param  {Integer}  i a menu item index
             * @return {Boolean}  true if the given menu item is decorative
             * @method isDecorative
             */
            this.isDecorative = function(i){
                return this.kids[i].$isDecorative === true || this.kids[i].$$isDecorative === true;
            };

            /**
             * Define component events handler.
             * @param  {zebkit.ui.event.CompEvent} e  a component event
             * @method  childCompEnabled
             */
            this.childCompEnabled = this.childCompShown = function(e) {
                var src = e.source;
                for(var i = 0;i < this.kids.length; i++){
                    if (this.kids[i] === src) {
                        // clear selection if an item becomes not selectable
                        if (this.isItemSelectable(i) === false && (i === this.selectedIndex)) {
                            this.select(-1);
                        }
                        break;
                    }
                }
            };

            /**
             * Get a menu item by the given index
             * @param  {Integer} i a menu item index
             * @return {zebkit.ui.Panel} a menu item component
             * @method getMenuItemAt
             */
            this.getMenuItemAt = function(i) {
                if (zebkit.isString(i) === true) {
                    var item = this.byPath(i);
                    if (item !== null) {
                        return item;
                    }

                    for (var k in this.menus) {
                        item = this.menus[k].getMenuItemAt(i);
                        if (item !== null) {
                            return item;
                        }
                    }
                }
                return this.kids[i];
            };

            /**
             * Test if the menu has a selectable item
             * @return {Boolean} true if the menu has at least one selectable item
             * @method hasSelectableItems
             */
            this.hasSelectableItems = function(){
                for(var i = 0; i < this.kids.length; i++) {
                    if (this.isItemSelectable(i)) {
                        return true;
                    }
                }
                return false;
            };

            /**
             * Define pointer exited events handler
             * @param  {zebkit.ui.event.PointerEvent} e a pointer event
             * @method pointerExited
             */
            this.pointerExited = function(e){
                this.position.setOffset(null);
            };

            /**
             * Get a sub menu for the given menu item
             * @param  {Integer} index a menu item index
             * @return {zebkit.ui.Menu} a sub menu or null if no sub menu
             * is defined for the given menu item
             * @method getMenuAt
             */
            this.getMenuAt = function(index) {
                if (index < this.kids.length) {
                    var hash = this.kids[index].$hash$;
                    return this.menus.hasOwnProperty(hash) ? this.menus[hash] : null;
                } else {
                    return null;
                }
            };

            // TODO: not stable API
            this.menuById = function(id) {
                if (this.id === id) {
                    return this;
                } else {
                    for (var i = 0; i < this.kids.length; i++)  {
                        var m = this.getMenuAt(i);
                        if (m !== null) {
                            var res = m.menuById(id);
                            if (res !== null) {
                                return res;
                            }
                        }
                    }
                    return null;
                }
            };

            /**
             * Get menu item by id.
             * @param  {String} id an id.
             * @return {zebkit.ui.MenuItem} a menu item
             * @method menuItemById
             */
            this.menuItemById = function(id) {
                for (var i = 0; i < this.kids.length; i++)  {
                    var mi = this.kids[i];
                    if (mi !== null && mi.id === id) {
                        return mi;
                    } else {
                        var m = this.getMenuAt(i);
                        if (m !== null) {
                            var res = m.menuItemById(id);
                            if (res !== null) {
                                return res;
                            }
                        }
                    }
                }
                return null;
            };

            /**
             * Set the given menu as a sub-menu for the specified menu item
             * @param {Integer} i an index of a menu item for that a sub menu
             * has to be attached
             * @param {zebkit.ui.Menu} m a sub menu to be attached
             * @method setMenuAt
             * @chainable
             */
            this.setMenuAt = function (i, m) {
                if (m === this) {
                    throw new Error("Menu cannot be sub-menu of its own");
                }

                if (this.isDecorative(i)) {
                    throw new Error("Decorative element cannot have a sub-menu");
                }

                var p = this.kids[i];
                if (p.activateSub !== undefined) {
                    var sub = this.menus.hasOwnProperty(p) ? this.menus[p] : null;
                    if (m !== null) {
                        if (sub === null) {
                            p.activateSub(true);
                        }
                    } else if (sub !== null) {
                        p.activateSub(false);
                    }
                }

                // if the menu is shown and the menu item is selected
                if (this.parent !== null && i === this.selectedIndex) {
                    this.select(-1);
                }

                if (p.$hash$ === undefined) {
                    throw new Error("Invalid key");
                }

                if (m === null) {
                    delete this.menus[p];
                } else {
                    this.menus[p] = m;
                }

                return this;
            };

            /**
             * Get the specified sub-menu index
             * @param  {zebkit.ui.Menu} menu a sub menu
             * @return {Integer} a sub menu index. -1 if the menu is
             * not a sub menu of the given menu
             * @method indexMenuOf
             */
            this.indexMenuOf = function(menu) {
                for(var i = 0; i < this.kids.length; i++) {
                    if (this.menus[this.kids[i]] === menu) {
                        return i;
                    }
                }
                return -1;
            };

            /**
             * Called when the menu or a sub-menu has been canceled (key ESCAPE has been pressed).
             * @param  {zebkit.ui.Menu} m a menu (or sub menu) that has been canceled
             * @method $canceled
             * @protected
             */
            this.$canceled = function(m) {
                if (this.$parentMenu !== null && this.$canceled !== undefined) {
                    this.$parentMenu.$canceled(m);
                }
            };

            /**
             * Get the top menu in the given shown popup menu hierarchy
             * @return {zebkit.ui.Menu} a top menu
             * @method $topMenu
             * @protected
             */
            this.$topMenu = function() {
                if (this.parent !== null) {
                    var t = this,
                        p = null;

                    while ((p = t.$parentMenu) !== null) {
                        t = p;
                    }
                    return t;
                }
                return null;
            };

            this.doScroll = function(dx, dy, source) {
                var sy = this.scrollManager.getSY(),
                    ps = this.layout.calcPreferredSize(this),
                    eh = this.height - this.getTop() - this.getBottom();

                if (this.height < ps.height && sy + ps.height >= eh && sy - dy <= 0) {
                    var nsy = sy - dy;
                    if (nsy + ps.height < eh) {
                        nsy = eh - ps.height;
                    }
                    if (sy !== nsy) {
                        this.scrollManager.scrollYTo(nsy);
                    }
                }
            };

            /**
             * Hide the menu and all visible sub-menus
             * @method $hideMenu
             * @protected
             */
            this.$hideMenu = function() {
                if (this.parent !== null) {
                    var ch = this.$childMenu();
                    if (ch !== null) {
                        ch.$hideMenu();
                    }

                    this.removeMe();
                    this.select(-1);
                }
            };

            /**
             * Get a sub menu that is shown at the given moment.
             * @return {zebkit.ui.Menu} a child sub menu. null if no child sub-menu
             * has been shown
             * @method $childMenu
             * @protected
             */
            this.$childMenu = function() {
                if (this.parent !== null) {
                    for(var k in this.menus) {
                        var m = this.menus[k];
                        if (m.$parentMenu === this) {
                            return m;
                        }
                    }
                }
                return null;
            };

            /**
             * Show the given sub menu
             * @param  {zebkit.ui.Menu} sub a sub menu to be shown
             * @method $showSubMenu
             * @protected
             */
            this.$showSubMenu = function(sub) {
                sub.setLocation(this.x + this.width - 10,
                                this.y + this.kids[this.selectedIndex].y);
                sub.toPreferredSize();
                this.parent.add(sub);
                sub.requestFocus();
            };

            this.triggerSelectionByPos = function(i) {
                return this.getMenuAt(i) !== null && this.$triggeredByPointer === true;
            };
        },

        /**
         * Override key pressed events handler to handle key events according to
         * context menu component requirements
         * @param  {zebkit.ui.event.KeyEvent} e a key event
         * @method keyPressed
         */
        function keyPressed(e){
            if (e.code === "Escape") {
                if (this.parent !== null) {
                    var p = this.$parentMenu;
                    this.$canceled(this);
                    this.$hideMenu();
                    if (p !== null) {
                        p.requestFocus();
                    }
                }
            } else {
                this.$super(e);
            }
        },

        function insert(i, ctr, c) {
            if (zebkit.isString(c)) {
                return this.$super(i, ctr, (c.match(/^\-+$/) !== null) ? new this.clazz.Line()
                                                                       : new this.clazz.MenuItem(c));
            } else if (zebkit.instanceOf(c, pkg.Panel)) {
                return this.$super(i, ctr, c);
            } else {
                return this.$super(i, ctr, new this.clazz.MenuItem(c));
            }
        },

        function setParent(p) {
            if (p !== null) {
                this.select(-1);
                this.position.setOffset(null);
            } else {
                this.$parentMenu = null;
            }
            this.$super(p);
        },

        /**
         * Add the specified component as a decorative item of the menu
         * @param {zebkit.ui.Panel} c an UI component
         * @method addDecorative
         */
        function addDecorative(c) {
            if (c.$isDecorative !== true) {
                c.$$isDecorative = true;
            }
            this.$getSuper("insert").call(this, this.kids.length, null, c);
        },

        function kidRemoved(i, c, ctr) {
            if (c.$$isDecorative !== undefined) {
                delete c.$$isDecorative;
            }
            this.setMenuAt(i, null);
            this.$super(i, c, ctr);
        },

        function isItemSelectable(i) {
            return this.$super(i) && this.isDecorative(i) === false;
        },

        function posChanged(target,prevOffset,prevLine,prevCol) {
            var off = target.offset;

            if (off >= 0) {
                var rs = null;

                // hide previously shown sub menu if position has been re-newed
                if (this.selectedIndex >= 0  && off !== this.selectedIndex) {
                    var sub = this.getMenuAt(this.selectedIndex);
                    if (sub !== null) {
                        sub.$hideMenu();
                        rs = -1; // request to clear selection
                        this.requestFocus();
                    }
                }

                // request fire selection if the menu is shown and position has moved to new place
                if (this.parent !== null && off !== this.selectedIndex && this.isItemSelectable(off)) {
                    if (this.triggerSelectionByPos(off)) {
                        rs = off;
                    }
                }

                if (rs !== null) {
                    this.select(rs);
                }
            }

            this.$super(target, prevOffset, prevLine, prevCol);
        },

        function fireSelected(prev) {
            if (this.parent !== null) {
                var sub = null;

                if (this.selectedIndex >= 0) {
                    sub = this.getMenuAt(this.selectedIndex);
                    if (sub !== null) { // handle sub menu here
                        if (sub.parent !== null) {
                            // hide menu since it has been already shown
                            sub.$hideMenu();
                        } else {
                            // show menu
                            sub.$parentMenu = this;
                            this.$showSubMenu(sub);
                        }
                    } else {
                        // handle an item menu selection here.
                        // hide the whole menu hierarchy
                        var k = this.kids[this.selectedIndex];
                        if (k.itemSelected !== undefined) {
                            k.itemSelected();
                        }

                        pkg.events.fire("menuItemSelected",
                                        MENU_EVENT.$fillWith(this,
                                                             this.selectedIndex,
                                                             this.kids[this.selectedIndex]));


                        // an atomic menu, what means a menu item has been selected
                        // remove this menu an all parents menus
                        var top = this.$topMenu();
                        if (top !== null) {
                            top.$hideMenu();
                        }
                    }

                } else if (prev >= 0) {
                    // hide child menus if null item has been selected
                    sub = this.getMenuAt(prev);
                    if (sub !== null && sub.parent !== null) {
                        // hide menu since it has been already shown
                        sub.$hideMenu();
                    }
                }
            }
            this.$super(prev);
        }
    ]);

    /**
     * Menu bar UI component class. Menu bar can be build in any part of UI application.
     * There is no restriction regarding the placement of the component.
     *
     *      var canvas = new zebkit.ui.zCanvas(300,200);
     *      canvas.setBorderLayout();
     *
     *      var mbar = new zebkit.ui.Menubar([
     *          {
     *              content: "Item 1",
     *              sub    : [
     *                  "Subitem 1.1",
     *                  "Subitem 1.2",
     *                  "Subitem 1.3"
     *              ]
     *          },
     *          {
     *              content: "Item 2",
     *              sub: [
     *                  "Subitem 2.1",
     *                  "Subitem 2.2",
     *                  "Subitem 2.3"
     *              ]
     *          },
     *          {
     *              content: "Item 3"
     *          }
     *      ]);
     *
     *      canvas.root.add("bottom", mbar);
     *
     * @class zebkit.ui.Menubar
     * @constructor
     * @extends zebkit.ui.Menu
     */
    pkg.Menubar = Class(pkg.Menu, [
        function $clazz() {
            this.MenuItem = Class(pkg.MenuItem, [
                function(c) {
                    this.$super(c);
                    this.getSub().setVisible(false);
                    this.getCheck().setVisible(false);
                },

                function $prototype() {
                    this.$getCheckSize = function() {
                        return pkg.$getPS(this.getCheck());
                    };
                },

                function $clazz() {
                    this.Label = Class(pkg.MenuItem.Label, []);
                }
            ]);
        },

        function $prototype() {
            this.canHaveFocus = false;

            this.triggerSelectionByPos = function (i) {
                return this.isItemSelectable(i) && this.selectedIndex >= 0;
            };

            // making menu bar not removable from its layer by overriding the method
            this.$hideMenu = function() {
                var child = this.$childMenu();
                if (child !== null) {
                    child.$hideMenu();
                }

                this.select(-1);
            };

            this.$showSubMenu = function(menu) {
                var d   = this.getCanvas(),
                    k   = this.kids[this.selectedIndex],
                    pop = d.getLayer(pkg.PopupLayer.id);

                if (menu.hasSelectableItems()) {
                    var abs = zebkit.layout.toParentOrigin(0, 0, k);
                    menu.setLocation(abs.x, abs.y + k.height + 1);
                    menu.toPreferredSize();
                    pop.add(menu);
                    menu.requestFocus();
                }
            };

            this.$canceled = function(m) {
                this.select(-1);
            };
        },

        // called when an item is selected by user with pointer click or key
        function $select(i) {
            // if a user again pressed the same item consider it as
            // de-selection
            if (this.selectedIndex >= 0 && this.selectedIndex === i) {
                i = -1;
            }
            this.$super(i);
        }
    ]);

    pkg.PopupLayerLayout = Class(zebkit.layout.Layout, [
        function $prototype() {
            this.calcPreferredSize = function (target){
                return { width:0, height:0 };
            };

            this.doLayout = function(target) {
                for(var i = 0; i < target.kids.length; i++){
                    var m = target.kids[i];
                    if (zebkit.instanceOf(m, pkg.Menu)) {
                        var ps = m.getPreferredSize(),
                            xx = (m.x + ps.width  > target.width ) ? target.width  - ps.width  : m.x,
                            yy = (m.y + ps.height > target.height) ? target.height - ps.height : m.y;

                        m.setSize(ps.width, ps.height);
                        if (xx < 0) {
                            xx = 0;
                        }
                        if (yy < 0) {
                            yy = 0;
                        }
                        m.setLocation(xx, yy);
                    }
                }
            };
        }
    ]);

    /**
     * UI popup layer interface that defines common part of popup layer
     * implementation.
     * @class zebkit.ui.PopupLayerMix
     * @interface zebkit.ui.PopupLayerMix
     */
    pkg.PopupLayerMix = zebkit.Interface([
        function $clazz() {
            this.id = "popup";
        },

        function $prototype() {
            this.$prevFocusOwner = null;

            this.getFocusRoot = function() {
                return this;
            };

            this.childFocusGained = function(e) {
                if (zebkit.instanceOf(e.source, pkg.Menu)) {
                    if (e.related !== null && zebkit.layout.isAncestorOf(this, e.related) === false ) {
                        this.$prevFocusOwner = e.related;
                    }
                } else {
                    // means other than menu type of component grabs the focus
                    // in this case we should not restore focus when the popup
                    // component will be removed
                    this.$prevFocusOwner = null;
                }

                // save the focus owner whose owner was not a pop up layer
                if (e.related !== null && zebkit.layout.isAncestorOf(this, e.related) === false && zebkit.instanceOf(e.source, pkg.Menu)) {
                    this.$prevFocusOwner = e.related;
                }
            };

            this.isTriggeredWith = function(e) {
                return e.isAction() === false && (e.identifier === "rmouse" || e.touchCounter === 2);
            };

            /**
             * Define children components input events handler.
             * @param  {zebkit.ui.event.KeyEvent} e an input event
             * @method childKeyPressed
             */
            this.childKeyPressed = function(e){
                var p = e.source.$parentMenu;
                if (p !== undefined && p !== null) {
                    switch (e.code) {
                        case "ArrowRight" :
                            if (p.selectedIndex < p.model.count() - 1) {
                                p.requestFocus();
                                p.position.seekLineTo("down");
                            }
                            break;
                        case "ArrowLeft" :
                            if (p.selectedIndex > 0) {
                                p.requestFocus();
                                p.position.seekLineTo("up");
                            }
                            break;
                    }
                }
            };

            this.$topMenu = function() {
                if (this.kids.length > 0) {
                    for (var i = this.kids.length - 1; i >= 0; i--) {
                        if (zebkit.instanceOf(this.kids[i], pkg.Menu)) {
                            return this.kids[i].$topMenu();
                        }
                    }
                }
                return null;
            };

            this.compRemoved = function(e) {
                // if last component has been removed and the component is a menu
                // than try to restore focus owner
                if (this.$prevFocusOwner !== null && this.kids.length === 0 && zebkit.instanceOf(e.kid, pkg.Menu)) {
                    this.$prevFocusOwner.requestFocus();
                    this.$prevFocusOwner = null;
                }
            };

            this.pointerPressed = function(e) {
                if (this.kids.length > 0) {
                    var top = this.$topMenu();
                    if (top !== null) {
                        top.$hideMenu();
                    }

                    // still have a pop up components, than remove it
                    if (this.kids.length > 0) {
                        this.removeAll();
                    }

                    return true;
                } else {
                    return false;
                }
            };

            // show popup
            this.layerPointerClicked = function (e) {
                if (this.kids.length === 0 && this.isTriggeredWith(e)) {
                    var popup = null;
                    if (e.source.popup !== undefined && e.source.popup !== null) {
                        popup = e.source.popup;
                    } else if (e.source.getPopup !== undefined) {
                        popup = e.source.getPopup(e.source, e.x, e.y);
                    }

                    if (popup !== null) {
                        popup.setLocation(e.absX, e.absY);
                        this.add(popup);
                        popup.requestFocus();
                    }

                    return true;
                } else {
                    return false;
                }
            };
        },

        function getComponentAt(x, y) {
            // if there is a component on popup layer and the component is
            // not the popup layer itself than return the component otherwise
            // return null what delegates getComponentAt() to other layer
            if (this.kids.length > 0) {
                // if pressed has happened over a popup layer no a menu
                var cc = this.$super(x, y);
                if (cc === this) {
                    var top = this.$topMenu();
                    if (top !== null) {
                        // if top menu is menu bar. menu bar is located in other layer
                        // we need check if the pressed has happened not over the
                        // menu bar
                        if (zebkit.instanceOf(top, pkg.Menubar)) {
                            var origin = zebkit.layout.toParentOrigin(top);
                            // is pointer pressed inside menu bar
                            if (x >= origin.x && y >= origin.y && x < origin.x + top.width && y < origin.y + top.height) {
                                return null;
                            }
                        }
                    }
                }
                return cc;
            } else {
                return null;
            }
        }
    ]);

    /**
     * Simple popup layer implementation basing on "zebkit.ui.Panel" component.
     * @class zebkit.ui.PopupLayer
     * @extends zebkit.ui.Panel
     * @constructor
     * @uses zebkit.ui.PopupLayerMix
     */
    pkg.PopupLayer = Class(pkg.Panel, pkg.PopupLayerMix, []);
});
